/* MColorPopup.cpp - color popup for liblayout; displays a colored area, single click
 *                   brings up a window with a BColorControl; drag&drop is supported.
 *
 * Copyright (c) 2004 by Christian Packmann.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

#include "MColorPopup.h"
#include <math.h>

ColorWindow::ColorWindow(BPoint where, alignment align, rgb_color color, BHandler* target,
						 const char* title=NULL)
	: BWindow(BRect(where, BPoint(where.x+300, where.y+150)),
		title, B_TITLED_WINDOW,
		B_NOT_ZOOMABLE | B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{
	mTarget = target;
	if(NULL == title)
		SetTitle("Choose color");
	mColCtrl = new BColorControl(BPoint(0,0), B_CELLS_32x8, 4,
		"ColorControl", new BMessage(MSG_SET_COLOR), true);
	mColCtrl->SetTarget(target);
	mColCtrl->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	mColCtrl->SetValue(color);
	AddChild(mColCtrl);
	float w, h;
	mColCtrl->GetPreferredSize(&w, &h);
	ResizeTo(w, h);
	// Position window according to parameter
	switch(align) {
		case B_ALIGN_CENTER:
			where.x -= w/2;
			where.y -= h/2;
			MoveTo(where);
			break;
		case B_ALIGN_RIGHT:	// LEFT / RIGHT are ignored currently
		case B_ALIGN_LEFT:
			break;
	}
	Show();
}
bool ColorWindow::QuitRequested(void) {
	mTarget->Looper()->PostMessage(MSG_COLORWINDOW_CLOSE, mTarget);
	return true;
}
void ColorWindow::MessageReceived(BMessage* msg) {
	switch(msg->what) {
		case MSG_SET_COLOR: {
				const rgb_color* colPtr;
				ssize_t size;
				if(B_OK==msg->FindData("RGBColor", B_RGB_COLOR_TYPE, (const void **)&colPtr, &size))
					mColCtrl->SetValue(*colPtr);
			}
			break;
		default:
			inherited::MessageReceived(msg);
	}
}
MColorPopup::MColorPopup(const char* label, rgb_color color, BMessage* msg,
			BHandler* target, const char* winTitle=NULL)
	: BControl(BRect(0,0,10,10), NULL, label, msg, B_FOLLOW_NONE, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE | B_NAVIGABLE),
	  mColor(color)
{
	mColorWindow = NULL;
	mLabel = label;
	mWindowTitle = winTitle;
	mDivideSameAs = 0;
	mTarget = target;
	//SetTarget(target);
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
}
MColorPopup::~MColorPopup() {
	if(NULL != mColorWindow)
		mColorWindow->PostMessage(B_QUIT_REQUESTED);
}
minimax MColorPopup::layoutprefs(void) {
	GetFontHeight(&mFontHeight);
	ct_mpm.mini.x = 2*kOuterXSpacing + kColorfieldXMin;
	ct_mpm.mini.y = 2*kOuterYSpacing + kColorfieldYMin;
	if(NULL != mLabel) {
		ct_mpm.mini.x += StringWidth(mLabel) + kLabelSpacing;
		float height = ceilf(mFontHeight.ascent) + ceilf(mFontHeight.descent);
		ct_mpm.mini.y = max_c(ct_mpm.mini.y, height);
	}
	ct_mpm.maxi.x = 1e5;
	ct_mpm.maxi.y = 1e5;
	mpm = ct_mpm;
	return ct_mpm;
}
BRect MColorPopup::layout(BRect size) {
	ResizeTo(size.Width(), size.Height());
	MoveTo(size.LeftTop());
	mHitArea = Bounds();
	mHitArea.InsetBy(4,4); // actual colored area
	return size;
}
void MColorPopup::Draw(BRect updateRect) {
	BRect r = Bounds();
	r.InsetBy(1,1);
	if(IsFocus())
		SetHighColor(keyboard_navigation_color());
	else
		SetHighColor(ViewColor());
	StrokeRect(r);
	r.InsetBy(1,1);
	//r.InsetBy(2,2);
	DrawRaisedFrame(r);
	r.InsetBy(2, 2);
	SetHighColor(mColor);
	FillRect(r);
}
void MColorPopup::AttachedToWindow() {
	// If target is a view, the SetTarget() call won't work properly in
	// ctor - thus we set it here.
	SetTarget(mTarget);
	if(NULL == Target()) {
		SetTarget(Window());
	}
	SetEventMask(EventMask() & ~(B_POINTER_EVENTS));
}
void MColorPopup::MouseDown(BPoint oldPos) {
	BPoint newPos;
	uint32 buttons;
	GetMouse(&newPos, &buttons);
	
	if( mHitArea.Contains(oldPos) && ((buttons & B_SECONDARY_MOUSE_BUTTON) || (buttons & B_TERTIARY_MOUSE_BUTTON)))
	{
		if(!mColorWindow) 
			mColorWindow = new ColorWindow(ConvertToScreen(Bounds().RightTop()), B_ALIGN_RIGHT,
					mColor, this, mWindowTitle);
		else
			mColorWindow->Activate(true);
	}
	else {
		mIsDragging = true;
		mDragStartPoint = oldPos;
	}
}
void MColorPopup::MouseMoved(BPoint where, uint32 transit, const BMessage *msg) {
	// check if drag started
	if(mIsDragging) {
		BPoint p = mDragStartPoint - where;
		if(fabsf(p.x)>2 || fabsf(p.y>2)) {
			BMessage m;
			m.AddData("RGBColor", B_RGB_COLOR_TYPE, &mColor, sizeof(rgb_color));
			DragMessage(&m, mHitArea);
			mIsDragging = false;
		}
	}
}
void MColorPopup::MouseUp(BPoint pos) {
	if(mIsDragging) {
		if(!mColorWindow)
			mColorWindow = new ColorWindow(ConvertToScreen(Bounds().RightTop()), B_ALIGN_RIGHT,
					mColor, this, mWindowTitle);
		else
			mColorWindow->Activate(true);
	}
	mIsDragging = false;
}
void MColorPopup::KeyDown(const char *bytes, int32 numBytes) {
	BPoint p;
	uint32 buttons;
	switch(bytes[0]) {
		case B_ENTER:
		case B_SPACE:
			GetMouse(&p, &buttons, false);
			mColorWindow = new ColorWindow(ConvertToScreen(p), B_ALIGN_CENTER,
				mColor, this, mWindowTitle);
			break;
		default:
			BControl::KeyDown(bytes, numBytes);
			break;
	}
}
void MColorPopup::MessageReceived(BMessage *msg) {
	if(msg->WasDropped()) {
		const rgb_color* colPtr;
		int32 size;
		if(B_OK == msg->FindData("RGBColor", B_RGB_COLOR_TYPE, (const void**)&colPtr, &size))
		{
			mColor = *colPtr;
			// send off message
			if(NULL!=Target()) {
				BMessage m;
				if(NULL!=Message())
					m = *Message();
				m.AddInt64("when", system_time());
				m.AddPointer("source", this);
				uint32 col;
				col = ((uint32)mColor.red << 24) | ((uint32)mColor.green << 16) | ((uint32)mColor.blue << 8);
				m.AddInt32("be:value", col);
				m.AddData("RGBColor", B_RGB_COLOR_TYPE, &mColor,sizeof(rgb_color));
				m.AddMessenger("be:sender", Messenger());
				Invoke(&m);
			}
			Draw(Bounds());
		}
	}
	switch(msg->what) {
		case MSG_COLORWINDOW_CLOSE:
			mColorWindow = NULL;
			break;
		case MSG_SET_COLOR: {
				uint32 col;
				if(B_OK == msg->FindInt32("be:value", (int32*)&col)) {
					// update internal state
					mColor.red = (col&0xff000000) >> 24;
					mColor.green = (col&0x00ff0000) >> 16;
					mColor.blue = (col&0x0000ff00) >> 8;
					// send off message
					if(NULL!=Target()) {
						BMessage m;
						if(NULL!=Message())
							m = *Message();
						m.AddInt64("when", system_time());
						m.AddPointer("source", this);
						m.AddInt32("be:value", col);
						m.AddData("RGBColor", B_RGB_COLOR_TYPE, &mColor,sizeof(rgb_color));
						m.AddMessenger("be:sender", Messenger());
						Invoke(&m);
					}
					// redraw
					Draw(Bounds());
				}
			}
			break;
		default:
			BView::MessageReceived(msg);
	}
}


void MColorPopup::DivideSameAs(MDividable *dividable) {
	mDivideSameAs = dividable->LabelWidth();
	// ...
}
float MColorPopup::LabelWidth() {
	if(0 == mDivideSameAs)
		return mLabelWidth;
	else
		return mDivideSameAs;
}


void MColorPopup::DrawRaisedFrame(BRect frame) {
	SetHighColor(96, 96, 96);
	MovePenTo(frame.LeftBottom());
	StrokeLine(frame.RightBottom());
	StrokeLine(frame.RightTop());
	//SetHighColor(216,216,216);
	SetHighColor(220,220,220);
	StrokeLine(frame.LeftTop());
	StrokeLine(frame.LeftBottom());
	frame.InsetBy(1,1);
	SetHighColor(184, 184, 184);
	MovePenTo(frame.LeftBottom());
	StrokeLine(frame.RightBottom());
	StrokeLine(frame.RightTop());
	SetHighColor(255,255,255);
	StrokeLine(frame.LeftTop());
	StrokeLine(frame.LeftBottom());
}
void MColorPopup::UpdateInternalState() {
	if(NULL != mColorWindow) {
		// update color in ColorWindow
		BMessage* msg = new BMessage(MSG_SET_COLOR);
		msg->AddData("RGBColor", B_RGB_COLOR_TYPE, &mColor, sizeof(mColor));
		mColorWindow->PostMessage(msg);
	}
	Draw(Bounds());
}
rgb_color MColorPopup::Color() {
	return mColor;
}
uint8 MColorPopup::Red() {
	return mColor.red;
}
uint8 MColorPopup::Green() {
	return mColor.green;
}
uint8 MColorPopup::Blue() {
	return mColor.blue;
}
void MColorPopup::SetColor(rgb_color col) {
	mColor = col;
	UpdateInternalState();
}
void MColorPopup::SetRed(uint8 r) {
	mColor.red = r;
	UpdateInternalState();
}
void MColorPopup::SetGreen(uint8 g) {
	mColor.green = g;
	UpdateInternalState();
}
void MColorPopup::SetBlue(uint8 b) {
	mColor.blue = b;
	UpdateInternalState();
}
