/* WindowShadeGUI.cpp - simple GUI for Marco Nelissens WindowShade.
 *
 * Copyright (c) 2004 by Christian Packmann.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

#include "MApplication.h"
#include "MWindow.h"
#include "VGroup.h"
#include "HGroup.h"
#include "MBorder.h"
#include "MButton.h"
#include "MStringView.h"
#include "Space.h"
#include <interface/Screen.h>
#include <interface/Alert.h>
#include <app/Message.h>
#include <storage/Entry.h>
#include <storage/Path.h>
#include <storage/File.h>
#include <storage/FindDirectory.h>
#include <storage/FilePanel.h>
#include <stdio.h>
#include <iostream>

#include "WindowShadeGUI.h"
#include "MColorPopup.h"

void WSGUIPrefs::Load(const entry_ref *er, bool loadAll) {
	BFile file;
	file.SetTo(er, B_READ_ONLY);
	if(B_OK != file.InitCheck()) {
		((WSGUIApp*)be_app)->ErrorAlert("Can't read file.");
	}
	else {
		BMessage msg;
		if(B_OK == msg.Unflatten(&file)) {
			WSGUIPrefs prefs;
			bool error = false;
			const void *data;
			ssize_t size;
			//BRect winFrame;
			if(B_OK == msg.FindData(PREFS_ACTIVE_TAB, B_RGB_COLOR_TYPE, &data, &size))
				prefs.mRGBActiveTab = *((rgb_color *)data);
			else
				error = true;
			if(B_OK == msg.FindData(PREFS_ACTIVE_TITLE, B_RGB_COLOR_TYPE, &data, &size))
				prefs.mRGBActiveTitle = *((rgb_color *)data);
			else
				error = true;
			if(B_OK == msg.FindData(PREFS_ACTIVE_BORDER, B_RGB_COLOR_TYPE, &data, &size))
				prefs.mRGBActiveBorder = *((rgb_color *)data);
			else
				error = true;
			if(B_OK == msg.FindData(PREFS_INACTIVE_TITLE, B_RGB_COLOR_TYPE, &data, &size))
				prefs.mRGBInactiveTitle = *((rgb_color *)data);
			else
				error = true;
			if(B_OK == msg.FindData(PREFS_INACTIVE_BORDER, B_RGB_COLOR_TYPE, &data, &size))
				prefs.mRGBInactiveBorder = *((rgb_color *)data);
			else
				error = true;
			if(B_OK == msg.FindData(PREFS_FRAME, B_RGB_COLOR_TYPE, &data, &size))
				prefs.mRGBFrame = *((rgb_color *)data);
			else
				error = true;
			if(loadAll) {
				if(B_OK != msg.FindRect("win_frame", &(prefs.mWindowFrame)))
					error = true;
			}
			// Only apply loaded settings if all values could be read.
			if(error)
				((WSGUIApp*)be_app)->ErrorAlert("Invalid settings file.");
			else {
				// copy simple vars
				mRGBActiveTab = prefs.mRGBActiveTab;
				mRGBActiveTitle = prefs.mRGBActiveTitle;
				mRGBActiveBorder = prefs.mRGBActiveBorder;
				mRGBInactiveTitle = prefs.mRGBInactiveTitle;
				mRGBInactiveBorder = prefs.mRGBInactiveBorder;
				mRGBFrame = prefs.mRGBFrame;
				// window position - take care it fits onto screen
				if(loadAll) {
					mWindowFrame = prefs.mWindowFrame;
					BScreen scr;
					const int kWinWidth = 250;
					const int kWinHeight = 210;
					if(!scr.IsValid())
						mWindowFrame.Set(0, 0, kWinWidth, kWinHeight);
					else {
						BRect scrFrame(scr.Frame());
						// first set default size/position; if we have valid prefs, we overwrite
						// the default settings.
						float x,y;
						x = (scr.Frame().Width() - kWinWidth) / 2;
						y = (scr.Frame().Height() - kWinHeight) / 2;
						mWindowFrame.Set(x, y, x + kWinWidth, y + kWinHeight);
						
						// if frame from settings is okay, use that
						if(prefs.mWindowFrame.IsValid() && scrFrame.Contains(prefs.mWindowFrame))
							mWindowFrame = prefs.mWindowFrame;
					}
				} // if(loadAll)
			}
		} // if(B_OK == msg.Unflatten(&file))
		else
			((WSGUIApp*)be_app)->ErrorAlert("Invalid settings file.");
	}
}
void WSGUIPrefs::Save(const entry_ref *er, bool saveAll) {
	BFile file;
	file.SetTo(er, B_READ_WRITE | B_CREATE_FILE);
	if(B_OK == file.InitCheck()) {
		BMessage msg;
		if(saveAll) {
			msg.AddRect("win_frame", mWindowFrame);
		}
		//WSGUIPrefs* prefs = &mPrefs;
		msg.AddData(PREFS_ACTIVE_TAB, B_RGB_COLOR_TYPE, &(mRGBActiveTab), 4);
		msg.AddData(PREFS_ACTIVE_TITLE, B_RGB_COLOR_TYPE, &(mRGBActiveTitle), 4);
		msg.AddData(PREFS_ACTIVE_BORDER, B_RGB_COLOR_TYPE, &(mRGBActiveBorder), 4);
		msg.AddData(PREFS_INACTIVE_TITLE, B_RGB_COLOR_TYPE, &(mRGBInactiveTitle), 4);
		msg.AddData(PREFS_INACTIVE_BORDER, B_RGB_COLOR_TYPE, &(mRGBInactiveBorder), 4);
		msg.AddData(PREFS_FRAME, B_RGB_COLOR_TYPE, &(mRGBFrame), 4);
		if(saveAll) {
		}
		if(B_OK != msg.Flatten(&file))
			((WSGUIApp*)be_app)->ErrorAlert("Can't write settings file.");
	}
}


WSGUIWindow::WSGUIWindow(BRect frame)
	: MWindow(frame, "WindowShadeGUI", B_TITLED_WINDOW, B_NOT_ZOOMABLE)
{
	// init vars
	mLoadPanel = 0;
	mSavePanel = 0;
	// create GUI
	MView *topView;
	WSGUIApp *app = (WSGUIApp*)be_app;
	WSGUIPrefs *prefs = &(app->mPrefs);
	topView = new VGroup(
		new MBorder(M_NO_BORDER, 7, NULL,
			new MBorder(M_ETCHED_BORDER, 5, NULL,
				new VGroup(
					new HGroup(
						new MStringView("Active Tab"),
						mCPActiveTab = new MColorPopup(NULL, prefs->mRGBActiveTab, new BMessage(MSG_ACTIVE_TAB), this, "Active Tab"),
						NULL
					),
					new HGroup(
						new MStringView("Active Title"),
						mCPActiveTitle = new MColorPopup(NULL, prefs->mRGBActiveTitle, new BMessage(MSG_ACTIVE_TITLE), this, "Active Title"),
						NULL
					),
					new HGroup(
						new MStringView("Active Border"),
						mCPActiveBorder = new MColorPopup(NULL, prefs->mRGBActiveBorder, new BMessage(MSG_ACTIVE_BORDER), this, "Active Border"),
						NULL
					),
					new HGroup(
						new MStringView("Inactive Title"),
						mCPInactiveTitle = new MColorPopup(NULL, prefs->mRGBInactiveTitle, new BMessage(MSG_INACTIVE_TITLE), this, "Inactive Title"),
						NULL
					),
					new HGroup(
						new MStringView("Inactive Border"),
						mCPInactiveBorder = new MColorPopup(NULL, prefs->mRGBInactiveBorder, new BMessage(MSG_INACTIVE_BORDER), this, "Inactive Border"),
						NULL
					),
					new HGroup(
						new MStringView("Frame"),
						mCPFrame = new MColorPopup(NULL, prefs->mRGBFrame, new BMessage(MSG_FRAME), this, "Frame"),
						NULL
					),
					0
				) // VGroup
			) // MBorder
		), // MBorder
		new HGroup(
			mBTLoad = new MButton("Load Colors", new BMessage(MSG_LOAD), this),
			new Space(minimax(20,1,1e5,1e5,0.25)),
			mBTSave = new MButton("Save Colors", new BMessage(MSG_SAVE), this),
			NULL
		),
		//new Space(minimax(20,5,1e5,20,0.05)),
		
		new HGroup(
			mBTRevert = new MButton("Revert", new BMessage(MSG_REVERT), this),
			mBTDefaults = new MButton("Defaults", new BMessage(MSG_DEFAULTS), this),
			new Space(minimax(20,1,1e5,1e5,0.25)),
			mBTApply = new MButton("Apply", new BMessage(MSG_APPLY), this),
			NULL
		),
		0
	); // VGroup
	AddChild(dynamic_cast<BView*>(topView));
	Show();
}
WSGUIWindow::~WSGUIWindow() {
	WSGUIApp* app = (WSGUIApp*)be_app;
	BRect tmpFrame(Frame()); // store current window frame
	// Activate the last valid color set before closing (either colors from
	// launch, or last time Apply was hit).
	app->mPrefs = app->mPrefsCopy;
	CallWindowShade();
	// restore current window frame;
	app->mPrefs.mWindowFrame = tmpFrame;
}
bool WSGUIWindow::QuitRequested() {
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}
void WSGUIWindow::MessageReceived(BMessage *msg) {
	WSGUIApp *app = (WSGUIApp*)be_app;
	WSGUIPrefs *prefs = &(app->mPrefs);
	const rgb_color* colPtr;
	int32 size;
	switch(msg->what) {
		case MSG_APPLY:
			Apply();
			break;
		case MSG_REVERT:
			Revert();
			break;
		case MSG_DEFAULTS:
			prefs->SetToDefaults();
			UpdateColorsFromPrefs();
			CallWindowShade();
			break;
		case MSG_ACTIVE_TAB:
			if(B_OK==msg->FindData("RGBColor", B_RGB_COLOR_TYPE, (const void **)&colPtr, &size)) {
				prefs->mRGBActiveTab = *colPtr;
				CallWindowShade();
			}
			break;
		case MSG_ACTIVE_TITLE:
			if(B_OK==msg->FindData("RGBColor", B_RGB_COLOR_TYPE, (const void **)&colPtr, &size)) {
				prefs->mRGBActiveTitle = *colPtr;
				CallWindowShade();
			}
			break;
		case MSG_ACTIVE_BORDER:
			if(B_OK==msg->FindData("RGBColor", B_RGB_COLOR_TYPE, (const void **)&colPtr, &size)) {
				prefs->mRGBActiveBorder = *colPtr;
				CallWindowShade();
			}
			break;
		case MSG_INACTIVE_TITLE:
			if(B_OK==msg->FindData("RGBColor", B_RGB_COLOR_TYPE, (const void **)&colPtr, &size)) {
				prefs->mRGBInactiveTitle = *colPtr;
				CallWindowShade();
			}
			break;
		case MSG_INACTIVE_BORDER:
			if(B_OK==msg->FindData("RGBColor", B_RGB_COLOR_TYPE, (const void **)&colPtr, &size)) {
				prefs->mRGBInactiveBorder = *colPtr;
				CallWindowShade();
			}
			break;
		case MSG_FRAME:
			if(B_OK==msg->FindData("RGBColor", B_RGB_COLOR_TYPE, (const void **)&colPtr, &size)) {
				prefs->mRGBFrame = *colPtr;
				CallWindowShade();
			}
			break;
		case MSG_LOAD:
			if(NULL == mLoadPanel) {
				mLoadPanel = new BFilePanel(B_OPEN_PANEL);
				//mLoadPanel->SetTarget(be_app_messenger);
			}
			mLoadPanel->Show();
			break;
		case MSG_SAVE:
			if(NULL == mSavePanel) {
				mSavePanel = new BFilePanel(B_SAVE_PANEL);
			}
			mSavePanel->Show();
			break;
		default:
			inherited::MessageReceived(msg);
	}
}
void WSGUIWindow::UpdateColorsFromPrefs(void) {
	WSGUIApp *app = (WSGUIApp*)be_app;
	mCPActiveTab->SetColor(app->mPrefs.mRGBActiveTab);
	mCPActiveTitle->SetColor(app->mPrefs.mRGBActiveTitle);
	mCPActiveBorder->SetColor(app->mPrefs.mRGBActiveBorder);
	mCPInactiveTitle->SetColor(app->mPrefs.mRGBInactiveTitle);
	mCPInactiveBorder->SetColor(app->mPrefs.mRGBInactiveBorder);
	mCPFrame->SetColor(app->mPrefs.mRGBFrame);
}	
void WSGUIWindow::Apply() {
	WSGUIApp *app = (WSGUIApp*)be_app;
	app->mPrefsCopy = app->mPrefs;
}
void WSGUIWindow::Revert() {
	
	WSGUIApp *app = (WSGUIApp*)be_app;
	app->mPrefs = app->mPrefsCopy;
	UpdateColorsFromPrefs();
	CallWindowShade();
}
void WSGUIWindow::CallWindowShade(void) {
	// get md5sum of app_server
	char md5SumString[256];
	FILE *md5Output = popen("md5sum /boot/beos/system/servers/app_server", "r");
	if(NULL == md5Output)
		fprintf(stderr, "Error executing md5sum.\n");
	else {
		fscanf(md5Output, "%256s", md5SumString);
		fclose(md5Output);
	}
	// Prepare command string for calling WindowShade
	WSGUIApp *app = (WSGUIApp*)be_app;
	WSGUIPrefs *prefs = &(app->mPrefs);
	char cmdString[256];
	sprintf(cmdString, "WindowShade -sum %s -activetab %02x%02x%02x -activetitle %02x%02x%02x -activeborder %02x%02x%02x -inactivetitle %02x%02x%02x -inactiveborder %02x%02x%02x -frame %02x%02x%02x -refresh\n",
		md5SumString,
		prefs->mRGBActiveTab.red, prefs->mRGBActiveTab.green, prefs->mRGBActiveTab.blue,
		prefs->mRGBActiveTitle.red, prefs->mRGBActiveTitle.green, prefs->mRGBActiveTitle.blue,
		prefs->mRGBActiveBorder.red, prefs->mRGBActiveBorder.green, prefs->mRGBActiveBorder.blue,
		prefs->mRGBInactiveTitle.red, prefs->mRGBInactiveTitle.green, prefs->mRGBInactiveTitle.blue,
		prefs->mRGBInactiveBorder.red, prefs->mRGBInactiveBorder.green, prefs->mRGBInactiveBorder.blue,
		prefs->mRGBFrame.red, prefs->mRGBFrame.green, prefs->mRGBFrame.blue
	);
	system(cmdString);
	// Write shell script to ~/config/bin
	BPath path;
	if(B_OK != find_directory(B_USER_CONFIG_DIRECTORY, &path))
		((WSGUIApp*)be_app)->ErrorAlert("Can't find users config directory.");
	else {
		path.Append("bin/SetWindowShadeColors.sh");
		BFile f(path.Path(), B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
		if(B_OK == f.InitCheck()) {
			const char* batchInitString = "#!/bin/sh\n";
			f.Write(batchInitString, strlen(batchInitString));
			f.Write(cmdString, strlen(cmdString));
			mode_t perms;
			f.GetPermissions(&perms);
			perms |= S_IXUSR;
			f.SetPermissions(perms);
		}
	}
}


WSGUIApp::WSGUIApp() : MApplication("application/x-vnd.CPackmann.WindowShadeGUI") {
	mPrefs.SetToDefaults(); // set default color values
	LoadPrefs();
	mPrefsCopy = mPrefs;
	mWSGUIWindow = new WSGUIWindow(mPrefs.mWindowFrame);
}
WSGUIApp::~WSGUIApp() {
	SavePrefs();
}
void WSGUIApp::LoadPrefs() {
	BPath path;
	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	path.Append(PREFS_FILE_NAME);
	if(B_OK == path.InitCheck()) {
		entry_ref er;
		if(B_OK == get_ref_for_path(path.Path(), &er))
			mPrefs.Load(&er);
	}
	mWSGUIWindow->CallWindowShade();
}
void WSGUIApp::SavePrefs() {
	BPath path;
	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	path.Append(PREFS_FILE_NAME);
	if(B_OK == path.InitCheck()) {
		entry_ref er;
		if(B_OK == get_ref_for_path(path.Path(), &er))
			mPrefs.Save(&er);
	}
}
void WSGUIApp::ErrorAlert(const char* alertText) {
	BAlert* alert = new BAlert("WindowShadeGUI error", alertText, 
			"Continue", NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING, B_WARNING_ALERT);
	alert->SetShortcut(0, B_ESCAPE);
	alert->Go();
}
void WSGUIApp::MessageReceived(BMessage *msg) {
	switch(msg->what) {
		case B_SAVE_REQUESTED: {
			entry_ref ref;
			const char* filename;
			if(B_OK == msg->FindRef("directory", &ref)) {
				if(B_OK == msg->FindString("name", &filename)) {
					BEntry entry(&ref, true);
					BPath path(&entry);
					path.Append(filename);
					if(B_OK == get_ref_for_path(path.Path(), &ref)) {
						mPrefs.Save(&ref, false);
					}
				}
			}
			break;
		}
		case B_QUIT_REQUESTED:
			
			Quit();
			break;
		default:
			MApplication::MessageReceived(msg);
	}
}
void WSGUIApp::RefsReceived(BMessage* msg) {
	entry_ref ref;
	if(B_OK == msg->FindRef("refs", &ref)) {
		mPrefsCopy = mPrefs; // copy old settings
		mPrefs.Load(&ref, false);
		// update window
		mWSGUIWindow->Lock();
		mWSGUIWindow->UpdateColorsFromPrefs();
		mWSGUIWindow->Unlock();
		mWSGUIWindow->CallWindowShade();
	}
}


int main(void) {
	WSGUIApp app;
	app.Run();
	return 0;
}
